@@ -29,11 +29,14 @@
 
 #ifdef _MSC_VER
 #define BELLESIP_INLINE __inline
+typedef signed char int8_t;
+typedef unsigned char uint8_t;
+typedef __int16 int16_t;
+typedef unsigned __int16 uint16_t;
 typedef __int32 int32_t;
 typedef unsigned __int32 uint32_t;
 typedef __int64 int64_t;
 typedef unsigned __int64 uint64_t;
-typedef unsigned char uint8_t;
 #else
 #include <inttypes.h>
 #define BELLESIP_INLINE inline
