@@ -23,7 +23,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
-#include "defs.h"
+#include "belle-sip/defs.h"
 
 
 BELLE_SIP_BEGIN_DECLS
@@ -157,6 +157,7 @@ BELLESIP_EXPORT void belle_sip_set_log_handler(belle_sip_log_function_t func);
 BELLESIP_EXPORT char * BELLE_SIP_CHECK_FORMAT_ARGS(1,2) belle_sip_strdup_printf(const char *fmt,...);
 
 BELLESIP_EXPORT belle_sip_error_code BELLE_SIP_CHECK_FORMAT_ARGS(4,5) belle_sip_snprintf(char *buff, size_t buff_size, size_t *offset, const char *fmt, ...);
+BELLESIP_EXPORT belle_sip_error_code belle_sip_snprintf_valist(char *buff, size_t buff_size, size_t *offset, const char *fmt, va_list args);
 
 BELLESIP_EXPORT void belle_sip_set_log_level(int level);
 
@@ -175,6 +176,10 @@ BELLESIP_EXPORT const char* belle_sip_version_to_string();
 **/
 BELLESIP_EXPORT char *belle_sip_unquote_strdup(const char *str);
 
+BELLESIP_EXPORT uint64_t belle_sip_time_ms(void);
+
+BELLESIP_EXPORT unsigned int belle_sip_random(void);
+
 #if defined(WIN32)
 
 #include <winsock2.h>
@@ -193,6 +198,15 @@ typedef int belle_sip_fd_t;
 
 #endif
 
+BELLESIP_EXPORT int belle_sip_getaddrinfo(const char *node, const char *service, const struct addrinfo *hints, struct addrinfo **res);
+BELLESIP_EXPORT void belle_sip_freeaddrinfo(struct addrinfo *res);
+
+
+typedef void (*belle_sip_background_task_end_callback_t)(void *);
+BELLESIP_EXPORT unsigned long belle_sip_begin_background_task(const char *name, belle_sip_background_task_end_callback_t cb, void *data);
+BELLESIP_EXPORT void belle_sip_end_background_task(unsigned long id);
+
+
 BELLE_SIP_END_DECLS
 
 #endif
