@@ -44,11 +44,14 @@ struct belle_sip_dns_srv{
 	unsigned short priority;
 	unsigned short weight;
 	unsigned short port;
+	unsigned char a_done;
+	unsigned char pad;
+	int cumulative_weight; /*used only temporarily*/
 	char *target;
 	belle_sip_combined_resolver_context_t *root_resolver;/* used internally to combine SRV and A queries*/
 	belle_sip_resolver_context_t *a_resolver; /* used internally to combine SRV and A queries*/
 	struct addrinfo *a_results; /* used internally to combine SRV and A queries*/
-	unsigned char a_done;
+	
 };
 
 static void belle_sip_dns_srv_destroy(belle_sip_dns_srv_t *obj){
@@ -61,7 +64,7 @@ static void belle_sip_dns_srv_destroy(belle_sip_dns_srv_t *obj){
 		obj->a_resolver=NULL;
 	}
 	if (obj->a_results){
-		freeaddrinfo(obj->a_results);
+		belle_sip_freeaddrinfo(obj->a_results);
 		obj->a_results=NULL;
 	}
 }
@@ -149,6 +152,7 @@ struct belle_sip_dual_resolver_context{
 
 void belle_sip_resolver_context_init(belle_sip_resolver_context_t *obj, belle_sip_stack_t *stack){
 	obj->stack=stack;
+	belle_sip_init_sockets(); /* Need to be called for DNS resolution to work on Windows platform. */
 }
 
 static struct dns_resolv_conf *resconf(belle_sip_simple_resolver_context_t *ctx) {
@@ -229,8 +233,13 @@ static struct dns_resolv_conf *resconf(belle_sip_simple_resolver_context_t *ctx)
 			if (ns_addr->sa_family==AF_INET6) using_ipv6=TRUE;
 		}
 		ctx->resconf->iface.ss_family=using_ipv6 ? AF_INET6 : AF_INET;
+		if (i==0) {
+			belle_sip_error("- no DNS servers available - resolution aborted.");
+			return NULL;
+		}
 	}else{
 		belle_sip_error("Error loading dns server addresses.");
+		return NULL;
 	}
 
 	return ctx->resconf;
@@ -243,8 +252,9 @@ static struct dns_hosts *hosts(belle_sip_simple_resolver_context_t *ctx) {
 		return ctx->hosts;
 
 	if (!(ctx->hosts = dns_hosts_local(&error))) {
-		belle_sip_error("%s dns_hosts_local error: %s", __FUNCTION__, dns_strerror(error));
-		return NULL;
+		belle_sip_warning("%s dns_hosts_local error: %s", __FUNCTION__, dns_strerror(error));
+		/*in case of failure, create an empty host object to make further processing happy, knowing that we can live without /etc/hosts.*/
+		ctx->hosts=dns_hosts_open(&error);
 	}
 
 	if (ctx->base.stack->dns_user_hosts_file) {
@@ -280,11 +290,98 @@ static int srv_compare_prio(const void *psrv1, const void *psrv2){
 	return 1;
 }
 
+/*
+ * see https://www.ietf.org/rfc/rfc2782.txt
+ * 0 weighted must just appear first.
+**/
+static int srv_sort_weight(const void *psrv1, const void *psrv2){
+	belle_sip_dns_srv_t *srv1=(belle_sip_dns_srv_t*)psrv1;
+	if (srv1->weight==0) return -1;
+	return 1;
+}
+
+static belle_sip_dns_srv_t *srv_elect_one(belle_sip_list_t *srv_list){
+	int sum=0;
+	belle_sip_list_t *elem;
+	belle_sip_dns_srv_t *srv;
+	int rand_number;
+	
+	for(elem=srv_list;elem!=NULL;elem=elem->next){
+		srv=(belle_sip_dns_srv_t*)elem->data;
+		sum+=srv->weight;
+		srv->cumulative_weight=sum;
+	}
+	/*no weights given, return the first one*/
+	if (sum==0) return (belle_sip_dns_srv_t*)srv_list->data;
+	rand_number=belle_sip_random() % sum; /*random number choosen in the range of the sum of weights*/
+	for(elem=srv_list;elem!=NULL;elem=elem->next){
+		srv=(belle_sip_dns_srv_t*)elem->data;
+		if (rand_number>=srv->cumulative_weight)
+			return srv;
+	}
+	return (belle_sip_dns_srv_t*)srv_list->data;
+}
+
+/*
+ * Order an SRV list with entries having the same priority according to their weight
+ */
+static belle_sip_list_t *srv_elect(belle_sip_list_t **srv_list) {
+	belle_sip_list_t *result = NULL;
+
+	while (*srv_list != NULL) {
+		belle_sip_list_t *it;
+		belle_sip_dns_srv_t *entry = srv_elect_one(*srv_list);
+		result = belle_sip_list_append(result, belle_sip_object_ref(entry));
+		it = belle_sip_list_find(*srv_list, entry);
+		if (it) {
+			*srv_list = belle_sip_list_remove_link(*srv_list, it);
+			belle_sip_free(it);
+		}
+	}
+	return result;
+}
+
+/*
+ * this function will return a list of SRV, with only one SRV record per priority.
+ */
+static belle_sip_list_t *srv_select_by_weight(belle_sip_list_t *srv_list){
+	belle_sip_list_t *same_prio=NULL;
+	belle_sip_list_t *elem;
+	belle_sip_dns_srv_t *prev_srv=NULL;
+	belle_sip_list_t *result=NULL;
+
+	for (elem=srv_list;elem!=NULL;elem=elem->next){
+		belle_sip_dns_srv_t *srv=(belle_sip_dns_srv_t*)elem->data;
+		if (prev_srv){
+			if (prev_srv->priority==srv->priority){
+				if (!same_prio){
+					same_prio=belle_sip_list_append(same_prio,prev_srv);
+				}
+				same_prio=belle_sip_list_insert_sorted(same_prio,srv,srv_sort_weight);
+			}else{
+				if (same_prio){
+					result=belle_sip_list_concat(result,srv_elect(&same_prio));
+				}
+			}
+		}
+		prev_srv=srv;
+	}
+	if (same_prio){
+		result=belle_sip_list_concat(result,srv_elect(&same_prio));
+	}
+	if (result){
+		belle_sip_list_free_with_data(srv_list,belle_sip_object_unref);
+		return result;
+	}
+	return srv_list;/*no weight election was necessary, return original list*/
+}
+
 static void notify_results(belle_sip_simple_resolver_context_t *ctx){
 	ctx->base.done=TRUE;
 	if ((ctx->type == DNS_T_A) || (ctx->type == DNS_T_AAAA)) {
 		ctx->cb(ctx->cb_data, ctx->name, ctx->ai_list);
 	} else if (ctx->type == DNS_T_SRV) {
+		ctx->srv_list=srv_select_by_weight(ctx->srv_list);
 		ctx->srv_cb(ctx->srv_cb_data, ctx->name, ctx->srv_list);
 	}
 }
@@ -491,10 +588,10 @@ struct addrinfo * belle_sip_ip_address_to_addrinfo(int family, const char *ipadd
 	hints.ai_flags=AI_NUMERICSERV|AI_NUMERICHOST;
 	hints.ai_socktype=SOCK_STREAM; //not used but it's needed to specify it because otherwise getaddrinfo returns one struct addrinfo per socktype.
 	
-	if (family==AF_INET6 && strchr(ipaddress,'.')!=NULL) {
+	if (family==AF_INET6 && strchr(ipaddress,':')==NULL) {
 		hints.ai_flags|=AI_V4MAPPED;
 	}
-	err=getaddrinfo(ipaddress,serv,&hints,&res);
+	err=belle_sip_getaddrinfo(ipaddress,serv,&hints,&res);
 
 	if (err!=0){
 		if (err!=EAI_NONAME)
@@ -520,7 +617,7 @@ static void belle_sip_combined_resolver_context_destroy(belle_sip_combined_resol
 }
 
 static void belle_sip_simple_resolver_context_destroy(belle_sip_simple_resolver_context_t *ctx){
-	/* Do not free elements of ctx->ai_list with freeaddrinfo(). Let the caller do it, otherwise
+	/* Do not free elements of ctx->ai_list with belle_sip_freeaddrinfo(). Let the caller do it, otherwise
 	   it will not be able to use them after the resolver has been destroyed. */
 	if (ctx->name != NULL) {
 		belle_sip_free(ctx->name);
@@ -550,11 +647,11 @@ static void belle_sip_dual_resolver_context_destroy(belle_sip_dual_resolver_cont
 		obj->aaaa_ctx=NULL;
 	}
 	if (obj->a_results){
-		freeaddrinfo(obj->a_results);
+		belle_sip_freeaddrinfo(obj->a_results);
 		obj->a_results=NULL;
 	}
 	if (obj->aaaa_results){
-		freeaddrinfo(obj->aaaa_results);
+		belle_sip_freeaddrinfo(obj->aaaa_results);
 		obj->aaaa_results=NULL;
 	}
 	if (obj->name){
@@ -714,10 +811,14 @@ static void process_a_from_srv(void *data, const char *name, struct addrinfo *ai
 static void srv_resolve_a(belle_sip_combined_resolver_context_t *obj, belle_sip_dns_srv_t *srv){
 	belle_sip_message("Starting A/AAAA query for srv result [%s]",srv->target);
 	srv->root_resolver=obj;
+	/* take a ref of the srv object because the A resolution may terminate synchronously and destroy the srv
+	 object before to store the returned value of belle_sip_stack_resole_a(). That would lead to an invalid write */
+	belle_sip_object_ref(srv);
 	srv->a_resolver=belle_sip_stack_resolve_a(obj->base.stack,srv->target,srv->port,obj->family,process_a_from_srv,srv);
 	if (srv->a_resolver){
 		belle_sip_object_ref(srv->a_resolver);
 	}
+	belle_sip_object_unref(srv);
 }
 
 static void process_srv_results(void *data, const char *name, belle_sip_list_t *srv_results){
@@ -726,11 +827,15 @@ static void process_srv_results(void *data, const char *name, belle_sip_list_t *
 	belle_sip_object_ref(ctx);
 	if (srv_results){
 		belle_sip_list_t *elem;
-		ctx->srv_results=srv_results;
+		/* take a ref of each srv_results because the last A resolution may terminate synchronously
+		 and destroy the list before the loop terminate */
+		ctx->srv_results = belle_sip_list_copy(srv_results);
+		belle_sip_list_for_each(srv_results, (void(*)(void *))belle_sip_object_ref);
 		for(elem=srv_results;elem!=NULL;elem=elem->next){
 			belle_sip_dns_srv_t *srv=(belle_sip_dns_srv_t*)elem->data;
 			srv_resolve_a(ctx,srv);
 		}
+		srv_results = belle_sip_list_free_with_data(srv_results, belle_sip_object_unref);
 	}else{
 		/*no SRV results, perform A query */
 		belle_sip_message("No SRV result for [%s], trying A/AAAA.",name);
@@ -873,15 +978,26 @@ void belle_sip_resolver_context_cancel(belle_sip_resolver_context_t *obj){
 	BELLE_SIP_OBJECT_VPTR(obj,belle_sip_resolver_context_t)->cancel(obj);
 }
 
-
+/*
+This function does the connect() method to get local ip address suitable to reach a given destination.
+It works on all platform except for windows using ipv6 sockets. TODO: find a workaround for win32+ipv6 socket
+*/
 void belle_sip_get_src_addr_for(const struct sockaddr *dest, socklen_t destlen, struct sockaddr *src, socklen_t *srclen, int local_port){
 	int af_type=dest->sa_family;
 	int sock=socket(af_type,SOCK_DGRAM,IPPROTO_UDP);
 	
+	belle_sip_message("belle_sip_get_src_addr_for(): af_inet6=%i",af_type==AF_INET6);
 	if (sock==(belle_sip_socket_t)-1){
 		belle_sip_fatal("Could not create socket: %s",belle_sip_get_socket_error_string());
 		goto fail;
 	}
+	
+	if (af_type==AF_INET6 && (IN6_IS_ADDR_V4MAPPED(&((struct sockaddr_in6*)dest)->sin6_addr))){
+		/*this is actually required only for windows, who is enable to provide an ipv4 mapped local address if the remote is ipv4 mapped,
+		and unable to provide a correct local address if the remote address is true ipv6 address when in dual stack mode*/
+		belle_sip_socket_enable_dual_stack(sock);
+	}
+	
 	if (connect(sock,dest,destlen)==-1){
 		belle_sip_error("belle_sip_get_src_addr_for: connect() failed: %s",belle_sip_get_socket_error_string());
 		goto fail;
@@ -907,7 +1023,7 @@ fail:
 		if (res != NULL) {
 			memcpy(src,res->ai_addr,MIN((size_t)*srclen,res->ai_addrlen));
 			*srclen=res->ai_addrlen;
-			freeaddrinfo(res);
+			belle_sip_freeaddrinfo(res);
 		} else {
 			belle_sip_fatal("belle_sip_get_src_addr_for(): belle_sip_ip_address_to_addrinfo() failed");
 		}
