@@ -18,13 +18,21 @@
 
 
 #define _CRT_RAND_S
+#include <stddef.h>
 #include <stdlib.h>
-
+#include <sys/stat.h>
 #include "belle_sip_internal.h"
 
-#include <time.h>
 #include "clock_gettime.h" /*for apple*/
 
+#ifndef WIN32
+#include <unistd.h>
+#include <sys/time.h> /*for gettimeofday*/
+#include <dirent.h> /* available on POSIX system only */
+#else
+#include <direct.h>
+#endif
+
 static FILE *__log_file=0;
 
 /**
@@ -111,10 +119,18 @@ char *belle_sip_strdup_printf(const char *fmt,...){
 }
 
 belle_sip_error_code belle_sip_snprintf(char *buff, size_t buff_size, size_t *offset, const char *fmt, ...) {
-	int ret;
-	belle_sip_error_code error = BELLE_SIP_OK;
+	belle_sip_error_code ret;
 	va_list args;
 	va_start(args, fmt);
+	ret = belle_sip_snprintf_valist(buff, buff_size, offset, fmt, args);
+	va_end(args);
+
+	return ret;
+}
+
+belle_sip_error_code belle_sip_snprintf_valist(char *buff, size_t buff_size, size_t *offset, const char *fmt, va_list args) {
+	int ret;
+	belle_sip_error_code error = BELLE_SIP_OK;
 	ret = vsnprintf(buff + *offset, buff_size - *offset, fmt, args);
 	if ((ret < 0)
 		|| (ret >= (buff_size - *offset))) {
@@ -123,7 +139,6 @@ belle_sip_error_code belle_sip_snprintf(char *buff, size_t buff_size, size_t *of
 	} else {
 		*offset += ret;
 	}
-	va_end(args);
 	return error;
 }
 
@@ -136,54 +151,88 @@ belle_sip_error_code belle_sip_snprintf(char *buff, size_t buff_size, size_t *of
 #if     defined(WIN32) || defined(_WIN32_WCE)
 void belle_sip_logv(int level, const char *fmt, va_list args)
 {
-        if (belle_sip_logv_out!=NULL && belle_sip_log_level_enabled(level))
-                belle_sip_logv_out(level,fmt,args);
-        if ((level)==BELLE_SIP_LOG_FATAL) abort();
+	if (belle_sip_logv_out!=NULL && belle_sip_log_level_enabled(level))
+		belle_sip_logv_out(level,fmt,args);
+	if ((level)==BELLE_SIP_LOG_FATAL) abort();
 }
 #endif
 
+
+#ifdef WIN32
+static int belle_sip_gettimeofday (struct timeval *tv, void* tz)
+{
+	union
+	{
+		__int64 ns100; /*time since 1 Jan 1601 in 100ns units */
+		FILETIME fileTime;
+	} now;
+
+	GetSystemTimeAsFileTime (&now.fileTime);
+	tv->tv_usec = (long) ((now.ns100 / 10LL) % 1000000LL);
+	tv->tv_sec = (long) ((now.ns100 - 116444736000000000LL) / 10000000LL);
+	return 0;
+}
+#else
+#define belle_sip_gettimeofday gettimeofday
+#endif
+
 static void __belle_sip_logv_out(belle_sip_log_level lev, const char *fmt, va_list args){
-        const char *lname="undef";
-        char *msg;
-        if (__log_file==NULL) __log_file=stderr;
-        switch(lev){
-                case BELLE_SIP_LOG_DEBUG:
-                        lname="debug";
-                        break;
-                case BELLE_SIP_LOG_MESSAGE:
-                        lname="message";
-                        break;
-                case BELLE_SIP_LOG_WARNING:
-                        lname="warning";
-                        break;
-                case BELLE_SIP_LOG_ERROR:
-                        lname="error";
-                        break;
-                case BELLE_SIP_LOG_FATAL:
-                        lname="fatal";
-                        break;
-                default:
-                        belle_sip_fatal("Bad level !");
-        }
-        msg=belle_sip_strdup_vprintf(fmt,args);
+	const char *lname="undef";
+	char *msg;
+	struct timeval tp;
+	struct tm *lt;
+#ifndef WIN32
+	struct tm tmstorage;
+#endif
+	time_t curtime;
+
+	belle_sip_gettimeofday(&tp,NULL);
+	curtime=tp.tv_sec;
+#ifdef WIN32
+	lt = localtime(&curtime);
+#else
+	lt = localtime_r(&curtime,&tmstorage);
+#endif
+
+	if (__log_file==NULL) __log_file=stderr;
+	switch(lev){
+		case BELLE_SIP_LOG_DEBUG:
+			lname="debug";
+			break;
+		case BELLE_SIP_LOG_MESSAGE:
+			lname="message";
+			break;
+		case BELLE_SIP_LOG_WARNING:
+			lname="warning";
+			break;
+		case BELLE_SIP_LOG_ERROR:
+			lname="error";
+			break;
+		case BELLE_SIP_LOG_FATAL:
+			lname="fatal";
+			break;
+		default:
+			belle_sip_fatal("Bad level !");
+	}
+	msg=belle_sip_strdup_vprintf(fmt,args);
 #if defined(_MSC_VER) && !defined(_WIN32_WCE)
 	#ifndef _UNICODE
-        OutputDebugStringA(msg);
-        OutputDebugStringA("\r\n");
+	OutputDebugStringA(msg);
+	OutputDebugStringA("\r\n");
 	#else
-		{
-			int len=strlen(msg);
-			wchar_t *tmp=(wchar_t*)belle_sip_malloc((len+1)*sizeof(wchar_t));
-			mbstowcs(tmp,msg,len);
-			OutputDebugStringW(tmp);
-			OutputDebugStringW(L"\r\n");
-			belle_sip_free(tmp);
-		}
+	{
+		int len=strlen(msg);
+		wchar_t *tmp=(wchar_t*)belle_sip_malloc((len+1)*sizeof(wchar_t));
+		mbstowcs(tmp,msg,len);
+		OutputDebugStringW(tmp);
+		OutputDebugStringW(L"\r\n");
+		belle_sip_free(tmp);
+	}
 	#endif
 #endif
-        fprintf(__log_file,"belle-sip-%s-%s" ENDLINE,lname,msg);
-        fflush(__log_file);
-        free(msg);
+	fprintf(__log_file,"%i-%.2i-%.2i %.2i:%.2i:%.2i:%.3i belle-sip-%s-%s" ENDLINE,1900+lt->tm_year,lt->tm_mon+1,lt->tm_mday,lt->tm_hour,lt->tm_min,lt->tm_sec,(int)(tp.tv_usec/1000), lname,msg);
+	fflush(__log_file);
+	free(msg);
 }
 
 belle_sip_list_t* belle_sip_list_new(void *data){
@@ -263,14 +312,18 @@ belle_sip_list_t * belle_sip_list_free_with_data(belle_sip_list_t *list, void (*
 }
 
 
-belle_sip_list_t*  belle_sip_list_remove(belle_sip_list_t* first, void *data){
+belle_sip_list_t*  _belle_sip_list_remove(belle_sip_list_t* first, void *data, int warn_if_not_found){
 	belle_sip_list_t* it;
 	it=belle_sip_list_find(first,data);
 	if (it) return belle_sip_list_delete_link(first,it);
-	else {
+	else if (warn_if_not_found){
 		belle_sip_warning("belle_sip_list_remove: no element with %p data was in the list", data);
-		return first;
 	}
+	return first;
+}
+
+belle_sip_list_t*  belle_sip_list_remove(belle_sip_list_t* first, void *data){
+	return _belle_sip_list_remove(first, data, TRUE);
 }
 
 int belle_sip_list_size(const belle_sip_list_t* first){
@@ -295,12 +348,14 @@ void belle_sip_list_for_each2(const belle_sip_list_t* list, void (*func)(void *,
 }
 
 belle_sip_list_t * belle_sip_list_pop_front(belle_sip_list_t *list, void **front_data){
-	if (list==NULL){
+	belle_sip_list_t *front_elem=list;
+	if (front_elem==NULL){
 		*front_data=NULL;
 		return NULL;
 	}
-	*front_data=list->data;
-	list=belle_sip_list_remove_link(list,list);
+	*front_data=front_elem->data;
+	list=belle_sip_list_remove_link(list,front_elem);
+	belle_sip_free(front_elem);
 	return list;
 }
 
@@ -333,9 +388,9 @@ belle_sip_list_t* belle_sip_list_find(belle_sip_list_t* list, void *data){
 	return NULL;
 }
 
-belle_sip_list_t* belle_sip_list_find_custom(belle_sip_list_t* list, belle_sip_compare_func compare_func, const void *user_data){
+belle_sip_list_t* belle_sip_list_find_custom(const belle_sip_list_t* list, belle_sip_compare_func compare_func, const void *user_data){
 	for(;list!=NULL;list=list->next){
-		if (compare_func(list->data,user_data)==0) return list;
+		if (compare_func(list->data,user_data)==0) return (belle_sip_list_t *)list;
 	}
 	return NULL;
 }
@@ -541,10 +596,10 @@ uint64_t belle_sip_time_ms(void){
 }
 #else
 uint64_t belle_sip_time_ms(void){
-#ifdef WINAPI_FAMILY_PHONE_APP
-	return GetTickCount64();
-#else
+#if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
 	return GetTickCount();
+#else
+	return GetTickCount64();
 #endif
 }
 #endif
@@ -586,7 +641,7 @@ char* _belle_sip_str_dup_and_unquote_string(const char* quoted_string) {
 char *belle_sip_unquote_strdup(const char *str){
 	const char *p;
 	if (str==NULL) return NULL;
-	
+
 	for(p=str;*p!='\0';++p){
 		switch(*p){
 			case ' ':
@@ -607,7 +662,7 @@ char *belle_sip_unquote_strdup(const char *str){
 static int belle_sip_wincrypto_random(unsigned int *rand_number){
 	static HCRYPTPROV hProv=(HCRYPTPROV)-1;
 	static int initd=0;
-	
+
 	if (!initd){
 		if (!CryptAcquireContext(&hProv,NULL,NULL,PROV_RSA_FULL, CRYPT_VERIFYCONTEXT)){
 			belle_sip_error("Could not acquire a windows crypto context");
@@ -617,7 +672,7 @@ static int belle_sip_wincrypto_random(unsigned int *rand_number){
 	}
 	if (hProv==(HCRYPTPROV)-1)
 		return -1;
-	
+
 	if (!CryptGenRandom(hProv,4,(BYTE*)rand_number)){
 		belle_sip_error("CryptGenRandom() failed.");
 		return -1;
@@ -641,7 +696,7 @@ unsigned int belle_sip_random(void){
 	unsigned int ret;
 #ifdef _MSC_VER
 	/*rand_s() is pretty nice and simple function but is not wrapped by mingw.*/
-	
+
 	if (rand_s(&ret)==0){
 		return ret;
 	}
@@ -672,7 +727,7 @@ static const char *symbols="aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ
 char * belle_sip_random_token(char *ret, size_t size){
 	unsigned int val=0;
 	unsigned int i;
-	
+
 	for(i=0;i<size-1;++i){
 		if (i%5==0) val=belle_sip_random();
 		ret[i]=symbols[val & 63];
@@ -713,12 +768,12 @@ static int bits_reader_read(bits_reader_t *reader, int count, unsigned int *ret)
 	size_t byte_index=reader->bit_index/8;
 	size_t bit_index=reader->bit_index % 8;
 	int shift=32-bit_index-count;
-	
+
 	if (count>=24){
 		belle_sip_error("This bit reader cannot read more than 24 bits at once.");
 		return -1;
 	}
-	
+
 	if (byte_index<reader->buf_size)
 		tmp=((unsigned int)reader->buffer[byte_index++])<<24;
 	else{
@@ -731,7 +786,7 @@ static int bits_reader_read(bits_reader_t *reader, int count, unsigned int *ret)
 		tmp|=((unsigned int)reader->buffer[byte_index++])<<8;
 	if (byte_index<reader->buf_size)
 		tmp|=((unsigned int)reader->buffer[byte_index++]);
-	
+
 	tmp=tmp>>shift;
 	tmp=tmp & ((1<<count)-1);
 	reader->bit_index+=count;
@@ -742,9 +797,9 @@ static int bits_reader_read(bits_reader_t *reader, int count, unsigned int *ret)
 char * belle_sip_octets_to_text(const uint8_t *hash, size_t hash_len, char *ret, size_t size){
 	int i;
 	bits_reader_t bitctx;
-	
+
 	bits_reader_init(&bitctx,hash,hash_len);
-	
+
 	for(i=0;i<(int)size-1;++i){
 		unsigned int val=0;
 		if (bits_reader_read(&bitctx,6,&val)==0){
@@ -787,7 +842,7 @@ char* belle_sip_to_unescaped_string(const char* buff) {
 	char *output_buff=belle_sip_malloc(strlen(buff)+1);
 	unsigned int i;
 	unsigned int out_buff_index=0;
-	
+
 	for(i=0; buff[i]!='\0'; out_buff_index++) {
 		i+=belle_sip_get_char(buff+i,3,output_buff+out_buff_index);
 	}
@@ -929,20 +984,26 @@ static const char *get_sip_uri_header_noescapes() {
 }
 
 static char* belle_sip_escape(const char* buff, const char *noescapes) {
-	char output_buff[BELLE_SIP_MAX_TO_STRING_SIZE];
-	unsigned int i;
-	unsigned int out_buff_index=0;
-	for(i=0; buff[i] != '\0' && out_buff_index < sizeof(output_buff)-4; i++) {
-		/*-4 to make sure last param can be stored in escaped form*/
-		const char c = buff[i];
-		if (noescapes[(unsigned int) c] == 1) {
+	size_t outbuf_size=strlen(buff);
+	size_t orig_size=outbuf_size;
+	char *output_buff=(char*)belle_sip_malloc(outbuf_size+1);
+	int i;
+	int out_buff_index=0;
+
+	for(i=0; buff[i] != '\0'; i++) {
+		int c = ((unsigned char*)buff)[i];
+		if (outbuf_size<out_buff_index-3){
+			outbuf_size+=MAX(orig_size/2,3);
+			output_buff=belle_sip_realloc(output_buff,outbuf_size+1);
+		}
+		if (noescapes[c] == 1) {
 			output_buff[out_buff_index++]=c;
 		} else {
-			out_buff_index+=sprintf(output_buff+out_buff_index,"%%%02x", c);
+			out_buff_index+=snprintf(output_buff+out_buff_index,outbuf_size-out_buff_index,"%%%02x", c);
 		}
 	}
 	output_buff[out_buff_index]='\0';
-	return belle_sip_strdup(output_buff);
+	return output_buff;
 }
 
 char* belle_sip_uri_to_escaped_username(const char* buff) {
@@ -1036,3 +1097,127 @@ char* belle_generic_uri_to_escaped_path(const char* buff) {
 	return belle_sip_escape(buff, get_generic_uri_path_noescapes());
 }
 
+char* belle_sip_string_to_backslash_less_unescaped_string(const char* buff) {
+	char *output_buff=belle_sip_malloc(strlen(buff)+1);
+	unsigned int i;
+	unsigned int out_buff_index=0;
+
+	for(i=0; buff[i] != '\0'; i++) {
+		if (buff[i] == '\\') {
+			i++;/*skip \*/
+		}
+		/*make sure to only remove one \ in case of \\*/
+		output_buff[out_buff_index++]=buff[i];
+	}
+	output_buff[out_buff_index]='\0';
+	return output_buff;
+}
+char* belle_sip_display_name_to_backslashed_escaped_string(const char* buff) {
+	char output_buff[BELLE_SIP_MAX_TO_STRING_SIZE];
+	unsigned int i;
+	unsigned int out_buff_index=0;
+	for(i=0; buff[i] != '\0' && out_buff_index < sizeof(output_buff)-2; i++) {
+		/*-3 to make sure last param can be stored in escaped form*/
+		const char c = buff[i];
+		if (c == '\"' || c == '\\') {
+			output_buff[out_buff_index++]='\\'; /*insert escape character*/
+		}
+		output_buff[out_buff_index++]=c;
+	}
+	output_buff[out_buff_index]='\0';
+	return belle_sip_strdup(output_buff);
+}
+
+belle_sip_list_t *belle_sip_parse_directory(const char *path, const char *file_type) {
+	belle_sip_list_t* file_list = NULL;
+#ifdef WIN32
+	WIN32_FIND_DATA FileData;
+	HANDLE hSearch;
+	BOOL fFinished = FALSE;
+	char szDirPath[1024];
+#ifdef UNICODE
+	wchar_t wszDirPath[1024];
+#endif
+
+	if (file_type == NULL) {
+		file_type = ".*";
+	}
+	snprintf(szDirPath, sizeof(szDirPath), "%s\\*%s", path, file_type);
+#ifdef UNICODE
+	mbstowcs(wszDirPath, szDirPath, sizeof(wszDirPath));
+	hSearch = FindFirstFileExW(wszDirPath, FindExInfoStandard, &FileData, FindExSearchNameMatch, NULL, 0);
+#else
+	hSearch = FindFirstFileExA(szDirPath, FindExInfoStandard, &FileData, FindExSearchNameMatch, NULL, 0);
+#endif
+	if (hSearch == INVALID_HANDLE_VALUE) {
+		belle_sip_message("No file (*%s) found in [%s] [%d].", file_type, szDirPath, (int)GetLastError());
+		return NULL;
+	}
+	snprintf(szDirPath, sizeof(szDirPath), "%s", path);
+	while (!fFinished) {
+		char szFilePath[1024];
+#ifdef UNICODE
+		char filename[512];
+		wcstombs(filename, FileData.cFileName, sizeof(filename));
+		snprintf(szFilePath, sizeof(szFilePath), "%s\\%s", szDirPath, filename);
+#else
+		snprintf(szFilePath, sizeof(szFilePath), "%s\\%s", szDirPath, FileData.cFileName);
+#endif
+		file_list = belle_sip_list_append(file_list, belle_sip_strdup(szFilePath));
+		if (!FindNextFile(hSearch, &FileData)) {
+			if (GetLastError() == ERROR_NO_MORE_FILES) {
+				fFinished = TRUE;
+			}
+			else {
+				belle_sip_error("Couldn't find next (*%s) file.", file_type);
+				fFinished = TRUE;
+			}
+		}
+	}
+	/* Close the search handle. */
+	FindClose(hSearch);
+#else
+	DIR *dir;
+	struct dirent *ent;
+	struct dirent *result;
+	long int name_max;
+	int res;
+
+	if ((dir = opendir(path)) == NULL) {
+		belle_sip_error("Could't open [%s] directory.", path);
+		return NULL;
+	}
+
+	/* allocate the directory entry structure */
+	name_max = pathconf(path, _PC_NAME_MAX);
+	if (name_max == -1) name_max = 255;
+	ent = malloc(offsetof(struct dirent, d_name) + name_max + 1);
+
+	/* loop on all directory files */
+	res = readdir_r(dir, ent, &result);
+	while ((res == 0) && (result != NULL)) {
+		/* filter on file type if given */
+		if (file_type==NULL
+			|| (strncmp(ent->d_name+strlen(ent->d_name)-strlen(file_type), file_type, strlen(file_type))==0) ) {
+			char *name_with_path=belle_sip_strdup_printf("%s/%s",path,ent->d_name);
+			file_list = belle_sip_list_append(file_list, name_with_path);
+		}
+		res = readdir_r(dir, ent, &result);
+	}
+	if (res != 0) {
+		belle_sip_error("Error while reading the [%s] directory: %s.", path, strerror(errno));
+	}
+	free(ent);
+	closedir(dir);
+#endif
+	return file_list;
+}
+
+int belle_sip_mkdir(const char *path) {
+#ifdef WIN32
+	return _mkdir(path);
+#else
+	return mkdir(path, 0700);
+#endif
+}
+
