@@ -26,6 +26,7 @@
 #endif
 
 #define belle_sip_network_buffer_size 65535
+#define belle_sip_send_network_buffer_size 16384
 
 typedef enum belle_sip_channel_state{
 	BELLE_SIP_CHANNEL_INIT,
@@ -41,38 +42,44 @@ typedef enum belle_sip_channel_state{
 const char * belle_sip_channel_state_to_string(belle_sip_channel_state_t state);
 
 /**
-* belle_sip_channel_t is an object representing a single communication channel ( socket or file descriptor), 
-* unlike the belle_sip_listening_point_t that can owns several channels for TCP or TLS (incoming server child sockets or 
+* belle_sip_channel_t is an object representing a single communication channel ( socket or file descriptor),
+* unlike the belle_sip_listening_point_t that can owns several channels for TCP or TLS (incoming server child sockets or
 * outgoing client sockets).
 **/
 typedef struct belle_sip_channel belle_sip_channel_t;
 
 BELLE_SIP_DECLARE_INTERFACE_BEGIN(belle_sip_channel_listener_t)
 void (*on_state_changed)(belle_sip_channel_listener_t *l, belle_sip_channel_t *, belle_sip_channel_state_t state);
-int (*on_event)(belle_sip_channel_listener_t *l, belle_sip_channel_t *obj, unsigned revents);
+void (*on_message_headers)(belle_sip_channel_listener_t *l, belle_sip_channel_t *obj, belle_sip_message_t *msg);
+void (*on_message)(belle_sip_channel_listener_t *l, belle_sip_channel_t *obj, belle_sip_message_t *msg);
 void (*on_sending)(belle_sip_channel_listener_t *l, belle_sip_channel_t *obj, belle_sip_message_t *msg);
 int (*on_auth_requested)(belle_sip_channel_listener_t *l, belle_sip_channel_t *obj, const char* distinghised_name);
 BELLE_SIP_DECLARE_INTERFACE_END
 
 #define BELLE_SIP_CHANNEL_LISTENER(obj) BELLE_SIP_INTERFACE_CAST(obj,belle_sip_channel_listener_t)
-#define MAX_CHANNEL_BUFF_SIZE 64000 + 1500 + 1
 
 typedef enum input_stream_state {
-	WAITING_MESSAGE_START=0
-	,MESSAGE_AQUISITION=1
-	,BODY_AQUISITION=2
+	WAITING_MESSAGE_START,
+	MESSAGE_AQUISITION,
+	BODY_AQUISITION
 }input_stream_state_t;
 
+typedef enum output_stream_state{
+	OUTPUT_STREAM_IDLE,
+	OUTPUT_STREAM_SENDING_HEADERS,
+	OUTPUT_STREAM_SENDING_BODY
+}output_stream_state_t;
+
 typedef struct belle_sip_channel_input_stream{
 	input_stream_state_t state;
-	char buff[MAX_CHANNEL_BUFF_SIZE];
+	char buff[belle_sip_network_buffer_size];
 	char* read_ptr;
 	char* write_ptr;
 	belle_sip_message_t *msg;
-	unsigned char *body; /*used when receiving a chunked body*/
 	int content_length;
 	int chuncked_mode;
 	int chunk_size;
+	int chunk_read_size;
 }belle_sip_channel_input_stream_t;
 
 typedef struct belle_sip_stream_channel belle_sip_stream_channel_t;
@@ -96,15 +103,23 @@ struct belle_sip_channel{
 	struct addrinfo *peer_list;
 	struct addrinfo *current_peer;
 	belle_sip_list_t *outgoing_messages;
-	belle_sip_list_t* incoming_messages;
+	belle_sip_message_t *cur_out_message;
+	output_stream_state_t out_state;
+	uint8_t *ewouldblock_buffer;
+	size_t ewouldblock_size;
+	size_t ewouldblock_offset;
 	belle_sip_channel_input_stream_t input_stream;
+	belle_sip_list_t* incoming_messages;
 	belle_sip_source_t *inactivity_timer;
 	uint64_t last_recv_time;
 	int simulated_recv_return; /* used to simulate network error. 0= no data (disconnected) >0= do nothing -1= network error*/
+	unsigned long bg_task_id;
+	unsigned long recv_bg_task_id;
 	unsigned char force_close; /* when channel is intentionnaly disconnected, in order to prevent looping notifications*/
 	unsigned char learnt_ip_port;
 	unsigned char has_name; /*set when the name of the peer is known, which is not the case of inboud connections*/
 	unsigned char about_to_be_closed;
+	unsigned char srv_overrides_port; /*set when this channel was connected to destination port provided by SRV resolution*/
 };
 
 #define BELLE_SIP_CHANNEL(obj)		BELLE_SIP_CAST(obj,belle_sip_channel_t)
@@ -149,11 +164,19 @@ int belle_sip_channel_is_reliable(const belle_sip_channel_t *obj);
 
 const char * belle_sip_channel_get_transport_name(const belle_sip_channel_t *obj);
 const char * belle_sip_channel_get_transport_name_lower_case(const belle_sip_channel_t *obj);
+char *belle_sip_channel_get_public_ip_port(belle_sip_channel_t *obj);
 
 const struct addrinfo * belle_sip_channel_get_peer(belle_sip_channel_t *obj);
 
 const char *belle_sip_channel_get_local_address(belle_sip_channel_t *obj, int *port);
 
+/*
+ * Returns a sip-uri suitable for using in record-route.
+ * If the channel is not natted, it will return the listening port of the listening point corresponding to the channel.
+**/
+belle_sip_uri_t *belle_sip_channel_create_routable_uri(belle_sip_channel_t *chan);
+
+
 #define belle_sip_channel_get_state(chan) ((chan)->state)
 
 void channel_set_state(belle_sip_channel_t *obj, belle_sip_channel_state_t state);
