@@ -30,14 +30,15 @@
 #include <arpa/inet.h>
 #include <netinet/tcp.h>
 #include <pthread.h>
+#include <strings.h>
 
 #else
 
 #include <winsock2.h>
 #include <ws2tcpip.h>
 
+#define strcasecmp _stricmp
 #ifdef _MSC_VER
-#define strcasecmp(a,b) _stricmp(a,b)
 #define snprintf _snprintf
 #define vsnprintf _vsnprintf
 #define strdup _strdup
@@ -57,6 +58,12 @@
 #	ifndef AI_V4MAPPED
 #	define AI_V4MAPPED 0x00000800
 #	endif
+#	ifndef AI_ALL
+#	define AI_ALL 0x00000100
+#	endif
+#	ifndef IPV6_V6ONLY
+#	define IPV6_V6ONLY 27
+#	endif
 #endif
 
 #if defined(WIN32) || defined(_WIN32) || defined(__WIN32) || defined(__WIN32__)
@@ -77,9 +84,15 @@ BELLESIP_INTERNAL_EXPORT int belle_sip_init_sockets(void);
 BELLESIP_INTERNAL_EXPORT void belle_sip_uninit_sockets(void);
 int belle_sip_socket_set_nonblocking (belle_sip_socket_t sock);
 int belle_sip_socket_set_dscp(belle_sip_socket_t sock, int ai_family, int dscp);
- 
+int belle_sip_socket_enable_dual_stack(belle_sip_socket_t sock);
+
 #if defined(WIN32)
 
+#if defined(__MINGW32__) || !defined(WINAPI_FAMILY_PARTITION)
+// Only use with x being WINAPI_PARTITION_DESKTOP to test if building on desktop
+#define WINAPI_FAMILY_PARTITION(x) 1
+#endif
+
 typedef HANDLE belle_sip_thread_t;
 
 #define belle_sip_thread_self_id()		(unsigned long)GetCurrentThreadId()
@@ -103,7 +116,7 @@ static BELLESIP_INLINE int get_socket_error(void){
 const char *belle_sip_get_socket_error_string();
 const char *belle_sip_get_socket_error_string_from_code(int code);
 
-#ifdef WINAPI_FAMILY_PHONE_APP
+#if !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
 BELLESIP_INTERNAL_EXPORT void belle_sip_sleep(unsigned int ms);
 #else
 #define belle_sip_sleep Sleep
@@ -120,6 +133,8 @@ static BELLESIP_INLINE int inet_aton(const char *ip, struct in_addr *p){
 
 #else
 
+#define WINAPI_FAMILY_PARTITION(x) 1
+
 typedef pthread_t belle_sip_thread_t;
 #define belle_sip_thread_self_id()			(unsigned long)pthread_self()
 #define belle_sip_thread_get_id(thread)			(unsigned long)thread
