@@ -46,8 +46,6 @@ int stream_channel_send(belle_sip_stream_channel_t *obj, const void *buf, size_t
 		int errnum=get_socket_error();
 		if (!belle_sip_error_code_is_would_block(errnum)){
 			belle_sip_error("Could not send stream packet on channel [%p]: %s",obj,belle_sip_get_socket_error_string());
-		}else{
-			belle_sip_warning("Channel [%p]: stream_channel_send EWOULDBLOCK",obj);
 		}
 		return -errnum;
 	}
@@ -71,7 +69,7 @@ int stream_channel_recv(belle_sip_stream_channel_t *obj, void *buf, size_t bufle
 void stream_channel_close(belle_sip_stream_channel_t *obj){
 	belle_sip_socket_t sock = belle_sip_source_get_socket((belle_sip_source_t*)obj);
 	if (sock!=(belle_sip_socket_t)-1){
-#ifdef TARGET_OS_IPHONE
+#if TARGET_OS_IPHONE
 		if (obj->read_stream != NULL) {
 			CFReadStreamClose (obj->read_stream);
 			CFRelease (obj->read_stream);
@@ -87,7 +85,7 @@ void stream_channel_close(belle_sip_stream_channel_t *obj){
 	}
 }
 
-#ifdef TARGET_OS_IPHONE
+#if TARGET_OS_IPHONE
 static void stream_channel_enable_ios_background_mode(belle_sip_stream_channel_t *obj){
 	int sock=belle_sip_source_get_socket((belle_sip_source_t*)obj);
 	
@@ -120,6 +118,7 @@ int stream_channel_connect(belle_sip_stream_channel_t *obj, const struct addrinf
 	belle_sip_socket_t sock;
 	tmp=1;
 	
+	obj->base.ai_family=ai->ai_family;
 	sock=socket(ai->ai_family, SOCK_STREAM, IPPROTO_TCP);
 	
 	if (sock==(belle_sip_socket_t)-1){
@@ -132,6 +131,9 @@ int stream_channel_connect(belle_sip_stream_channel_t *obj, const struct addrinf
 		belle_sip_error("setsockopt TCP_NODELAY failed: [%s]",belle_sip_get_socket_error_string());
 	}
 	belle_sip_socket_set_nonblocking(sock);
+	if (ai->ai_family==AF_INET6){
+		belle_sip_socket_enable_dual_stack(sock);
+	}
 	
 	err = connect(sock,ai->ai_addr,ai->ai_addrlen);
 	if (err != 0 && get_socket_error()!=BELLESIP_EINPROGRESS && get_socket_error()!=BELLESIP_EWOULDBLOCK) {
@@ -194,8 +196,10 @@ int finalize_stream_connection(belle_sip_stream_channel_t *obj, unsigned int rev
 #if TARGET_OS_IPHONE
 			stream_channel_enable_ios_background_mode(obj);
 #endif
-			if (obj->base.stack->dscp)
-				belle_sip_socket_set_dscp(sock,obj->base.lp->ai_family,obj->base.stack->dscp);
+			if (obj->base.stack->dscp && obj->base.lp){
+				/*apply dscp only to channel belonging to a SIP listening point*/
+				belle_sip_socket_set_dscp(sock,obj->base.ai_family,obj->base.stack->dscp);
+			}
 			set_tcp_nodelay(sock);
 			return 0;
 		}else{
