@@ -63,7 +63,7 @@ static int udp_channel_recv(belle_sip_channel_t *obj, void *buf, size_t buflen){
 }
 
 int udp_channel_connect(belle_sip_channel_t *obj, const struct addrinfo *ai){
-	struct sockaddr_storage laddr;
+	struct sockaddr_storage laddr={0};
 	socklen_t lslen=sizeof(laddr);
 	if (obj->local_ip==NULL){
 		belle_sip_get_src_addr_for(ai->ai_addr,ai->ai_addrlen,(struct sockaddr*)&laddr,&lslen,obj->local_port);
