@@ -28,11 +28,11 @@
 #define IPV4_CNAME		"stun.linphone.org"
 #define IPV4_CNAME_IP		"91.121.209.194"
 #define IPV4_SIP_BAD_DOMAIN	"dummy.linphone.org"
-#define IPV4_MULTIRES_DOMAIN	"google.com"
+#define IPV4_MULTIRES_DOMAIN	"yahoo.fr"
 
-/* videolan.org has an IPv6 and an IPv4 IP*/
-#define IPV6_SIP_DOMAIN		"videolan.org"
-#define IPV6_SIP_IP		"2a01:e0d:1:3:58bf:fa02:0:1"
+/* sip2.linphone.org has an IPv6 and an IPv4 IP*/
+#define IPV6_SIP_DOMAIN		"sip2.linphone.org"
+#define IPV6_SIP_IP		"2001:41d0:2:14b0::1"
 #define IPV6_SIP_IPV4		"88.191.250.2"
 
 #define SRV_DOMAIN		"linphone.org"
@@ -67,7 +67,6 @@ static unsigned int  wait_for(belle_sip_stack_t *stack, int *current_value, int
 
 static endpoint_t* create_endpoint() {
 	endpoint_t* endpoint;
-	if (belle_sip_init_sockets() < 0) return NULL;
 	endpoint = belle_sip_new0(endpoint_t);
 	endpoint->stack = belle_sip_stack_new(NULL);
 	return endpoint;
@@ -78,7 +77,7 @@ static void reset_endpoint(endpoint_t *endpoint) {
 	endpoint->resolve_done = 0;
 	endpoint->resolve_ko = 0;
 	if (endpoint->ai_list != NULL) {
-		freeaddrinfo(endpoint->ai_list);
+		belle_sip_freeaddrinfo(endpoint->ai_list);
 		endpoint->ai_list = NULL;
 	}
 	if (endpoint->srv_list != NULL) {
@@ -134,7 +133,7 @@ static void ipv4_a_query(void) {
 		ai = belle_sip_ip_address_to_addrinfo(AF_INET, IPV4_SIP_IP, SIP_PORT);
 		if (ai) {
 			CU_ASSERT_EQUAL(sock_in->sin_addr.s_addr, ((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr);
-			freeaddrinfo(ai);
+			belle_sip_freeaddrinfo(ai);
 		}
 	}
 
@@ -160,7 +159,7 @@ static void ipv4_cname_a_query(void) {
 		ai = belle_sip_ip_address_to_addrinfo(AF_INET, IPV4_CNAME_IP, SIP_PORT);
 		if (ai) {
 			CU_ASSERT_EQUAL(sock_in->sin_addr.s_addr, ((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr);
-			freeaddrinfo(ai);
+			belle_sip_freeaddrinfo(ai);
 		}
 	}
 
@@ -174,7 +173,12 @@ static void local_query(void) {
 	CU_ASSERT_PTR_NOT_NULL_FATAL(client);
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
 	client->resolver_ctx = belle_sip_stack_resolve_a(client->stack, "localhost", SIP_PORT, AF_INET, a_resolve_done, client);
-	CU_ASSERT_EQUAL(client->resolver_ctx, NULL);
+	// A DNS query is needed on Windows platform
+#ifdef WIN32
+	CU_ASSERT_PTR_NOT_EQUAL(client->resolver_ctx, NULL);
+#else
+	CU_ASSERT_PTR_EQUAL(client->resolver_ctx, NULL);
+#endif
 	CU_ASSERT_TRUE(wait_for(client->stack, &client->resolve_done, 1, timeout));
 	CU_ASSERT_PTR_NOT_EQUAL(client->ai_list, NULL);
 	if (client->ai_list) {
@@ -301,7 +305,7 @@ static void ipv6_aaaa_query(void) {
 			for (i = 0; i < 8; i++) {
 				CU_ASSERT_EQUAL(sock_in6->sin6_addr.s6_addr[i], ipv6_address->s6_addr[i]);
 			}
-			freeaddrinfo(ai);
+			belle_sip_freeaddrinfo(ai);
 		}
 		next=client->ai_list->ai_next;
 		CU_ASSERT_PTR_NOT_NULL(next);
@@ -318,7 +322,7 @@ static void ipv6_aaaa_query(void) {
 				for (i = 0; i < 8; i++) {
 					CU_ASSERT_EQUAL(sock_in6->sin6_addr.s6_addr[i], ipv6_address->s6_addr[i]);
 				}
-				freeaddrinfo(ai);
+				belle_sip_freeaddrinfo(ai);
 			}
 		}
 	}
@@ -378,7 +382,7 @@ static void srv_a_query_no_srv_result(void) {
 		ai = belle_sip_ip_address_to_addrinfo(AF_INET, IPV4_SIP_IP, SIP_PORT);
 		if (ai) {
 			CU_ASSERT_EQUAL(sock_in->sin_addr.s_addr, ((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr);
-			freeaddrinfo(ai);
+			belle_sip_freeaddrinfo(ai);
 		}
 	}
 
@@ -418,7 +422,7 @@ static void no_query_needed(void) {
 		ai = belle_sip_ip_address_to_addrinfo(AF_INET, IPV4_SIP_IP, SIP_PORT);
 		if (ai) {
 			CU_ASSERT_EQUAL(sock_in->sin_addr.s_addr, ((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr);
-			freeaddrinfo(ai);
+			belle_sip_freeaddrinfo(ai);
 		}
 	}
 
@@ -463,7 +467,7 @@ static void dns_fallback(void) {
 		ai = belle_sip_ip_address_to_addrinfo(AF_INET, IPV4_SIP_IP, SIP_PORT);
 		if (ai) {
 			CU_ASSERT_EQUAL(sock_in->sin_addr.s_addr, ((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr);
-			freeaddrinfo(ai);
+			belle_sip_freeaddrinfo(ai);
 		}
 	}
 
@@ -475,7 +479,7 @@ static void ipv6_dns_server(void) {
 	int timeout;
 	endpoint_t *client;
 	const char *nameservers[]={
-		"2a01:e00::2",
+		"2001:4860:4860::8888",
 		NULL
 	};
 	
@@ -498,7 +502,7 @@ static void ipv6_dns_server(void) {
 		ai = belle_sip_ip_address_to_addrinfo(AF_INET, IPV4_SIP_IP, SIP_PORT);
 		if (ai) {
 			CU_ASSERT_EQUAL(sock_in->sin_addr.s_addr, ((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr);
-			freeaddrinfo(ai);
+			belle_sip_freeaddrinfo(ai);
 		}
 	}
 
@@ -533,7 +537,7 @@ static void ipv4_and_ipv6_dns_server(void) {
 		ai = belle_sip_ip_address_to_addrinfo(AF_INET, IPV4_SIP_IP, SIP_PORT);
 		if (ai) {
 			CU_ASSERT_EQUAL(sock_in->sin_addr.s_addr, ((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr);
-			freeaddrinfo(ai);
+			belle_sip_freeaddrinfo(ai);
 		}
 	}
 
