@@ -65,9 +65,13 @@ extern const char * belle_sip_tester_test_suite_name(int suite_index);
 extern const char * belle_sip_tester_test_name(const char *suite_name, int test_index);
 extern void belle_sip_tester_init(void);
 extern void belle_sip_tester_uninit(void);
+extern const char * belle_sip_tester_get_root_ca_path(void);
+extern void belle_sip_tester_set_root_ca_path(const char *root_ca_path);
 extern int belle_sip_tester_run_tests(const char *suite_name, const char *test_name);
 
+extern const char *belle_sip_tester_writable_dir_prefix;
 extern const char* belle_sip_tester_client_cert;
+extern const char* belle_sip_tester_client_cert_fingerprint;
 extern const char* belle_sip_tester_private_key;
 extern const char* belle_sip_tester_private_key_passwd;
 
